/*
 * plymouth_funcs.c
 *
 * Plymouth (userspace splash) splash method support
 *
 * Copyright (c) 2009 Frederic Crozat <fcrozat@suse.com>
 *
 * This file is released under the GPLv2.
 *
 */

#include "config.h"

#ifdef CONFIG_PLYMOUTH
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdlib.h>

#include "splash.h"
#include "encrypt.h"
#include "plymouth_funcs.h"
#include <ply-event-loop.h>
#include <ply-boot-splash-plugin.h>
#include <ply-boot-splash.h>
//#include <ply-window.h>
#include <ply-progress.h>
#include <ply-utils.h>
#include <ply-terminal.h>
#include <ply-keyboard.h>
#include <ply-pixel-display.h>
#include <ply-text-display.h>
#include <ply-renderer.h>

static    ply_boot_splash_t *ply_splash;
static    ply_buffer_t *buffer;
//static    ply_window_t *window;
static    ply_keyboard_t *keyboard;
static    ply_terminal_t *terminal;
static    ply_progress_t *ply_progress;
static	  ply_renderer_t *renderer;
static	  ply_list_t *pixel_displays;
static	  ply_text_display_t *text_display;
static    int key_pressed = 0;
static	  char last_key = -1;

static void keyboard_input_handler (void *user_data, const char *keyboard_input, size_t      character_size) {
	fprintf(stderr, "key pressed\n");
	key_pressed = 1;
	if (character_size > 0) {
		last_key = keyboard_input[character_size - 1];
	}
}

int plymouth_update(const char *arg) {
	const char *cmd = "/usr/bin/plymouth";
	pid_t pid = fork();
	if (!pid) {
		int fd = open("/dev/null", O_RDWR);
		dup2(fd, 2);
		close(fd);
		if (execl(cmd, cmd, arg, (char *) NULL) == -1)
			exit(1);
	}
	else {
		int status;
		waitpid(pid, &status, 0);
		return status;
	}
	return 1;
}

void terminate_plymouth() {
        ply_keyboard_stop_watching_for_input (keyboard);
	ply_keyboard_remove_input_handler (keyboard, keyboard_input_handler); 
        ply_boot_splash_unset_keyboard (ply_splash);
	ply_boot_splash_hide (ply_splash);
	ply_boot_splash_free (ply_splash);
	ply_progress_free (ply_progress);
	ply_renderer_close (renderer);
	ply_renderer_free (renderer);
	ply_terminal_close (terminal);
	ply_terminal_free (terminal);
	ply_buffer_free (buffer);
}

static int plymouth_init() {
	
	int ret;
	ply_key_file_t *key_file;
	char *splash_string;
	char *splash_path;
	ply_list_t *heads;
	ply_list_node_t *node;


	buffer = ply_buffer_new ();

	key_file = ply_key_file_new (PLYMOUTH_CONFDIR "plymouthd.conf");

	if ((ret = ply_key_file_load (key_file)) == 0)
	{
		return ret;
	}
	splash_string = ply_key_file_get_value (key_file, "Daemon", "Theme");

	asprintf(&splash_path, PLYMOUTH_THEMESDIR "%s/%s.plymouth", splash_string, splash_string);
	free (splash_string);
	terminal =  ply_terminal_new ("tty63");
	ply_splash = ply_boot_splash_new (splash_path, PLYMOUTH_PLUGINSDIR, buffer);
	free (splash_path);
	if ((ret = ply_boot_splash_load (ply_splash)) == 0) {
		fprintf(stderr,"could not open splash\n");
		return ret;
	}
	renderer = ply_renderer_new (NULL, NULL, terminal);
	if ((ret = ply_renderer_open (renderer))== 0) {
		fprintf(stderr,"could not open renderer /dev/fb\n");
		return ret;
	}
	keyboard =  ply_keyboard_new_for_terminal (terminal);

	text_display = ply_text_display_new (terminal);
  	ply_boot_splash_add_text_display (ply_splash, text_display);

	heads = ply_renderer_get_heads (renderer);
	pixel_displays = ply_list_new();


	node = ply_list_get_first_node (heads);

	while (node != NULL)
        { 
	  ply_list_node_t *next_node;
          ply_renderer_head_t *head;
          ply_pixel_display_t *display;

          head = ply_list_node_get_data (node);
	  next_node = ply_list_get_next_node (heads, node);

	  display = ply_pixel_display_new (renderer, head);

	  ply_list_append_data (pixel_displays, display);
          node = next_node;
          ply_boot_splash_add_pixel_display (ply_splash, display);
         }

	ply_progress = ply_progress_new();
	ply_boot_splash_attach_progress (ply_splash, ply_progress);
	ply_boot_splash_attach_to_event_loop (ply_splash, ply_event_loop_get_default());


	ply_keyboard_add_input_handler(keyboard, keyboard_input_handler, NULL); 
	return ret;

}

int plymouth_open(int mode)
{
	int ret;

	if ((ret = plymouth_init ()) == 0 )
	{
		fprintf(stderr,"could not initialize plymouth : error %d \n",ret);
		return -1;
	}

	ply_boot_splash_set_keyboard (ply_splash, keyboard);
	ply_renderer_activate (renderer);
	if (mode==SPL_RESUME) {
		ply_boot_splash_show (ply_splash, PLY_BOOT_SPLASH_MODE_RESUME);
	}
	else {
		ply_boot_splash_show (ply_splash, PLY_BOOT_SPLASH_MODE_SUSPEND);
	       	ply_boot_splash_display_message (ply_splash, "Suspending..." );
	}
	ply_keyboard_watch_for_input(keyboard);

	ply_event_loop_process_pending_events (ply_event_loop_get_default());

	return 0;
}


inline int plymouth_finish(void)
{

	terminate_plymouth();

	return 0;
}

inline int plymouth_progress(int p)
{
	ply_progress_set_percentage (ply_progress, (double) p / 100);

	ply_event_loop_process_pending_events (ply_event_loop_get_default());

	return 0;
}


void plymouth_read_password(char *buf, int vrfy)
{
#if 0
#if CONFIG_ENCRYPT
        char *vrfy_buf = vrfy ? buf + PASS_SIZE : buf;

        do {
            while (plymouth_get_password(buf,PASS_SIZE, 
                        "Passphrase please (must be non-empty): ") <= 0);
            
            if (vrfy)
                while (plymouth_get_password(vrfy_buf,PASS_SIZE,
                            "Verify passphrase: ") <= 0);
            
    	} while (vrfy && strncmp(buf, vrfy_buf, PASS_SIZE));
#endif
#endif
}


void plymouth_set_caption(const char *message)
{
	ply_boot_splash_display_message (ply_splash, message);
	ply_event_loop_process_pending_events (ply_event_loop_get_default());
}

int plymouth_dialog(const char *prompt) 
{
	key_pressed = 0;
	ply_progress_pause (ply_progress);
       	ply_boot_splash_display_message (ply_splash, prompt);
	while (key_pressed == 0) {
		ply_event_loop_process_pending_events (ply_event_loop_get_default());
	}
	ply_progress_unpause (ply_progress);
	return last_key;
}

char plymouth_key_pressed(void) {
#if 1
	ply_event_loop_process_pending_events (ply_event_loop_get_default());
	if (key_pressed == 0) {
	       return 0;
	}
	else {
		int key = last_key;
		last_key = -1;
		key_pressed = 0;
		return key;
	}
#endif
//	return 0;
}


#endif
